# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['manageconf']

package_data = \
{'': ['*']}

install_requires = \
['anyconfig>=0.9.8,<0.10.0', 'boto3>=1.9,<2.0']

setup_kwargs = {
    'name': 'manageconf',
    'version': '1.0.0',
    'description': 'Builds a config object based on environment variables, settings files and (optional) parameters stored in AWS System Manager Parameter Store.',
    'long_description': '# Manage Conf\n\n[![CircleCI](https://circleci.com/gh/sam-atkins/manageconf/tree/main.svg?style=svg)](https://circleci.com/gh/sam-atkins/manageconf/tree/main)\n<a href="https://github.com/ambv/black"><img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n\n## Description\n\nBuilds a config object based on environment variables, settings files and (optional) parameters stored in AWS System Manager Parameter Store.\n\nThe config object merges in config, overriding any previous key/value pairs, in the following order:\n\n- ENV\n- default config: default.json\n- stage config: {stage}.json\n- remote config: remote_settings (AWS param store)\n\nAvailable to download as a package on [PyPi](https://pypi.org/project/manageconf/).\n\n### Settings Files\n\nSet an environment variable with the key name `project_config_dir`. It is important this is set before the package is imported. The value of `project_config_dir` should be the location of your `/settings` folder.\n\nSet-up your settings folder, adding in configuration to the appropriate file.\n\n```bash\n.\n├── settings                  <-- Settings folder\n│\xa0\xa0 ├── default.json          <-- default configuration\n│\xa0\xa0 ├── {stage}.json          <-- stage specific configuration e.g. `local`\n│\xa0\xa0 └── {stage}.json          <-- stage specific configuration e.g. `dev`\n```\n\nExample configuration:\n\n#### default.json\n```json\n{\n  "project_name": "example-project",\n  "DEBUG": "False"\n}\n```\n\n##### local.json\n```json\n{\n  "DEBUG": "True",\n  "use_remote_settings": "False"\n}\n```\n\nLocal config object:\n\n```python\n{\n    "project_name": "example-project",\n    "DEBUG": "True,\n    "use_remote_settings": "False"\n}\n```\n\n##### dev.json\n\n```json\n{\n  "use_remote_settings": "True"\n}\n```\n\nDev config object:\n\n```python\n{\n    "project_name": "example-project",\n    "DEBUG": "True",\n    "use_remote_settings": "True",\n    # and any remote settings from AWS param store\n}\n```\n\n\n### AWS\n\nAdd parameters in your AWS account with paths that match this pattern:\n\n`/{project_name}/{stage}/`\n\nIf you set `"use_remote_settings": "True"` in a remote `{stage}.json` config file, the package will attempt to fetch the parameters from the store that have this base path.\n\nUsing the example configuration above, the path would be:\n\n```\n/example-project/dev/\n```\n\n### Usage\n\nMake sure `project_config_dir` is set before importing the library.\n\n```python\nfrom manageconf import Config, get_config\n\nSECRET_KEY = get_config("SECRET_KEY")\nDEBUG = get_config("DEBUG", True)\n# default values are an optional second arg and will\n# be used if the param cannot be found in the config object\n```\n\n## Development\n\n### Install\n\nRequires [Poetry](https://poetry.eustace.io).\n\n```bash\n# create a Python3 virtual environment\nvirtualenv -p python3 env\n\n# activate the virtual env\nsource env/bin/activate\n\n# install requirements\npoetry install\n```\n\n### Tests\n\n```bash\n# run tests\npytest -vv\n\n# coverage report in the Terminal\npytest --cov=manageconf tests/\n\n# coverage report in HTML\npytest --cov-report html --cov=manageconf tests/\n```\n',
    'author': 'Sam Atkins',
    'author_email': 'samatkins@outlook.com',
    'url': 'https://github.com/sam-atkins/manageconf',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



__version__ = "1.0.0"

import ast
import json
import os

import anyconfig
import boto3

from manageconf.exceptions import (
    BotoRequestFailureError,
    RemoteConfigurationJSONDecodeError,
)


class RemoteSettings:
    """Methods to fetch and transform remote settings"""

    def _deserialise(self, name, value):
        """Deserialise JSON values to Python

        Args:
            name (str): the config key name
            value (str): the config value

        Returns:
            value: deserialised config value
        """
        if isinstance(value, str):
            try:
                return json.loads(value)
            except json.JSONDecodeError:
                raise RemoteConfigurationJSONDecodeError(
                    f"value of item {name} is not valid JSON"
                )
        else:
            return value

    def _evaluate(self, name, value):
        """Literal evaluation of a string containing a Python expression

        Args:
            name (str): the config key name
            value (str): the config value

        Returns:
            value: evaluated config value
        """
        if isinstance(value, str):
            try:
                return ast.literal_eval(value)
            except Exception:
                return value
        else:
            return value

    def get_remote_params(self, parameters_path):
        """Fetches remote config from AWS Systems Manager Param Store

        Args:
            parameters_path (str): the path of the params to fetch
                /{project_name}/{stage}/

        Returns:
            dict: remote config
        """
        client = boto3.client("ssm")
        response = {}
        try:
            payload = client.get_parameters_by_path(
                Path=parameters_path, Recursive=True, WithDecryption=True
            )

            remote_params = payload.get("Parameters", [])
            for param in remote_params:
                name = param.get("Name", None)
                name = name.split("/")[-1]
                value = param.get("Value", None)

                deserialised_value = self._deserialise(name, value)
                evaluated_value = self._evaluate(name, deserialised_value)
                response[name] = evaluated_value
            return response
        except Exception as ex:
            raise BotoRequestFailureError from ex(f"Boto response failed: {ex}")


class Config:
    """Merges in config objects to make one conf object"""

    conf = {}

    @classmethod
    def create_remote_settings_class(cls):
        setting_class = RemoteSettings

        return setting_class()

    @classmethod
    def make(cls):
        """Makes the conf object, merging in the following order:

            - ENV
            - default config: default.json
            - stage config: {stage}.json
            - remote config: remote_settings
        """
        anyconfig.merge(cls.conf, os.environ.copy())
        stage = cls.conf.get("stage", None)
        project_config_dir = cls.conf.get("project_config_dir", ".")

        project_default_config_file_path = os.path.join(
            project_config_dir, "default.json"
        )
        if os.path.exists(project_default_config_file_path):
            anyconfig.merge(cls.conf, anyconfig.load(project_default_config_file_path))

        project_stage_config_file_path = os.path.join(
            project_config_dir, f"{stage}.json"
        )
        if os.path.exists(project_stage_config_file_path):
            anyconfig.merge(cls.conf, anyconfig.load(project_stage_config_file_path))
        remote_settings = cls.conf.get("use_remote_settings", None)
        if remote_settings:
            project_name = cls.conf.get("project_name", None)
            parameters_path = f"/{project_name}/{stage}/"
            remote_settings_class = cls.create_remote_settings_class()
            remote_conf = remote_settings_class.get_remote_params(parameters_path)
            anyconfig.merge(cls.conf, remote_conf)


def get_config(key_name, default=None):
    return Config.conf.get(key_name, default)


def make_settings():
    Config.make()


make_settings()



class BotoRequestFailureError(Exception):
    """Raised when a Boto request to SSM fails"""


class RemoteConfigurationJSONDecodeError(Exception):
    """Raised when a Parameter Store item fails to be deserialised"""
